@@ -20,13 +20,14 @@
 #include <getopt.h>
 
 #include "volumes.h"
-#include "crc32c.h"
-#include "commands.h"
-#include "utils.h"
-#include "help.h"
+#include "crypto/crc32c.h"
+#include "cmds/commands.h"
+#include "common/utils.h"
+#include "common/help.h"
+#include "common/box.h"
 
 static const char * const btrfs_cmd_group_usage[] = {
-	"btrfs [--help] [--version] <group> [<group>...] <command> [<args>]",
+	"btrfs [--help] [--version] [--format <format>] [-v|--verbose] [-q|--quiet] <group> [<group>...] <command> [<args>]",
 	NULL
 };
 
@@ -42,10 +43,11 @@ static inline const char *skip_prefix(const char *str, const char *prefix)
 static int parse_one_token(const char *arg, const struct cmd_group *grp,
 			   const struct cmd_struct **cmd_ret)
 {
-	const struct cmd_struct *cmd = grp->commands;
 	const struct cmd_struct *abbrev_cmd = NULL, *ambiguous_cmd = NULL;
+	int i = 0;
 
-	for (; cmd->token; cmd++) {
+	for (i = 0; grp->commands[i]; i++) {
+		const struct cmd_struct *cmd = grp->commands[i];
 		const char *rest;
 
 		rest = skip_prefix(arg, cmd->token);
@@ -97,6 +99,19 @@ parse_command_token(const char *arg, const struct cmd_group *grp)
 	return cmd;
 }
 
+static void check_output_format(const struct cmd_struct *cmd)
+{
+	if (cmd->next)
+		return;
+
+	if (!(cmd->flags & bconf.output_format & CMD_FORMAT_MASK)) {
+		fprintf(stderr,
+			"ERROR: output format %s is unsupported for this command\n",
+			output_format_name(bconf.output_format));
+		exit(1);
+	}
+}
+
 static void handle_help_options_next_level(const struct cmd_struct *cmd,
 		int argc, char **argv)
 {
@@ -109,61 +124,115 @@ static void handle_help_options_next_level(const struct cmd_struct *cmd,
 			argv++;
 			help_command_group(cmd->next, argc, argv);
 		} else {
-			usage_command(cmd, 1, 0);
+			usage_command(cmd, true, false);
 		}
 
 		exit(0);
 	}
 }
 
-int handle_command_group(const struct cmd_group *grp, int argc,
+int handle_command_group(const struct cmd_struct *cmd, int argc,
 			 char **argv)
-
 {
-	const struct cmd_struct *cmd;
+	const struct cmd_struct *subcmd;
 
 	argc--;
 	argv++;
 	if (argc < 1) {
-		usage_command_group(grp, 0, 0);
+		usage_command_group(cmd->next, false, false);
 		exit(1);
 	}
 
-	cmd = parse_command_token(argv[0], grp);
+	subcmd = parse_command_token(argv[0], cmd->next);
 
-	handle_help_options_next_level(cmd, argc, argv);
+	handle_help_options_next_level(subcmd, argc, argv);
+	check_output_format(subcmd);
 
-	fixup_argv0(argv, cmd->token);
-	return cmd->fn(argc, argv);
+	fixup_argv0(argv, subcmd->token);
+	return cmd_execute(subcmd, argc, argv);
 }
 
 static const struct cmd_group btrfs_cmd_group;
 
 static const char * const cmd_help_usage[] = {
-	"btrfs help [--full]",
+	"btrfs help [--full] [--box]",
 	"Display help information",
 	"",
 	"--full     display detailed help on every command",
+	"--box      show list of built-in tools (busybox style)",
 	NULL
 };
 
-static int cmd_help(int argc, char **argv)
+static int cmd_help(const struct cmd_struct *unused, int argc, char **argv)
 {
+	int i;
+
+	for (i = 0; i < argc; i++) {
+		if (strcmp(argv[i], "--box") == 0) {
+#if ENABLE_BOX
+			printf("Standalone tools built-in in the busybox style:\n");
+			printf("- mkfs.btrfs\n");
+			printf("- btrfs-image\n");
+			printf("- btrfs-convert\n");
+			printf("- btrfstune\n");
+			printf("- btrfs-find-root\n");
+#else
+			printf("No standalone tools built-in in the busybox style\n");
+#endif
+			exit(0);
+		}
+	}
 	help_command_group(&btrfs_cmd_group, argc, argv);
 	return 0;
 }
 
+static DEFINE_SIMPLE_COMMAND(help, "help");
+
 static const char * const cmd_version_usage[] = {
 	"btrfs version",
 	"Display btrfs-progs version",
 	NULL
 };
 
-static int cmd_version(int argc, char **argv)
+static int cmd_version(const struct cmd_struct *unused, int argc, char **argv)
 {
 	printf("%s\n", PACKAGE_STRING);
 	return 0;
 }
+static DEFINE_SIMPLE_COMMAND(version, "version");
+
+static void print_output_formats(FILE *outf)
+{
+	int i;
+
+	fputs("Options for --format are:", outf);
+	for (i = 0; i < ARRAY_SIZE(output_formats); i++)
+		fprintf(outf, "%s%s", i ? ", " : " ", output_formats[i].name);
+	fputs("\n", outf);
+}
+
+static void handle_output_format(const char *format)
+{
+	int i;
+	bool found = false;
+
+	for (i = 0; i < ARRAY_SIZE(output_formats); i++) {
+		if (!strcasecmp(format, output_formats[i].name)) {
+			bconf.output_format = output_formats[i].value;
+			found = true;
+			break;
+		}
+	}
+
+	/* Print error for invalid format */
+	if (!found) {
+		bconf.output_format = CMD_FORMAT_TEXT;
+		fprintf(stderr, "error: invalid output format \"%s\"\n\n",
+			format);
+		print_output_formats(stderr);
+		exit(1);
+	}
+}
 
 /*
  * Parse global options, between binary name and first non-option argument
@@ -173,11 +242,14 @@ static int cmd_version(int argc, char **argv)
  */
 static int handle_global_options(int argc, char **argv)
 {
-	enum { OPT_HELP = 256, OPT_VERSION, OPT_FULL };
+	enum { OPT_HELP = 256, OPT_VERSION, OPT_FULL, OPT_FORMAT };
 	static const struct option long_options[] = {
 		{ "help", no_argument, NULL, OPT_HELP },
 		{ "version", no_argument, NULL, OPT_VERSION },
+		{ "format", required_argument, NULL, OPT_FORMAT },
 		{ "full", no_argument, NULL, OPT_FULL },
+		{ "verbose", no_argument, NULL, 'v' },
+		{ "quiet", no_argument, NULL, 'q' },
 		{ NULL, 0, NULL, 0}
 	};
 	int shift;
@@ -189,7 +261,7 @@ static int handle_global_options(int argc, char **argv)
 	while (1) {
 		int c;
 
-		c = getopt_long(argc, argv, "+", long_options, NULL);
+		c = getopt_long(argc, argv, "+vq", long_options, NULL);
 		if (c < 0)
 			break;
 
@@ -197,6 +269,15 @@ static int handle_global_options(int argc, char **argv)
 		case OPT_HELP: break;
 		case OPT_VERSION: break;
 		case OPT_FULL: break;
+		case OPT_FORMAT:
+			handle_output_format(optarg);
+			break;
+		case 'v':
+			bconf_be_verbose();
+			break;
+		case 'q':
+			bconf_be_quiet();
+			break;
 		default:
 			fprintf(stderr, "Unknown global option: %s\n",
 					argv[optind - 1]);
@@ -212,52 +293,61 @@ static int handle_global_options(int argc, char **argv)
 
 static void handle_special_globals(int shift, int argc, char **argv)
 {
-	int has_help = 0;
-	int has_full = 0;
+	bool has_help = false;
+	bool has_full = false;
 	int i;
 
 	for (i = 0; i < shift; i++) {
 		if (strcmp(argv[i], "--help") == 0)
-			has_help = 1;
+			has_help = true;
 		else if (strcmp(argv[i], "--full") == 0)
-			has_full = 1;
+			has_full = true;
 	}
 
 	if (has_help) {
 		if (has_full)
-			usage_command_group(&btrfs_cmd_group, 1, 0);
+			usage_command_group(&btrfs_cmd_group, true, false);
 		else
-			cmd_help(argc, argv);
+			cmd_execute(&cmd_struct_help, argc, argv);
+		print_output_formats(stdout);
 		exit(0);
 	}
 
 	for (i = 0; i < shift; i++)
 		if (strcmp(argv[i], "--version") == 0) {
-			cmd_version(argc, argv);
+			cmd_execute(&cmd_struct_version, argc, argv);
 			exit(0);
 		}
+
+	for (i = 0; i < shift; i++)
+		if (strcmp(argv[i], "--verbose") == 0)
+			bconf_be_verbose();
+
+	for (i = 0; i < shift; i++)
+		if (strcmp(argv[i], "--quiet") == 0)
+			bconf_be_quiet();
 }
 
 static const struct cmd_group btrfs_cmd_group = {
 	btrfs_cmd_group_usage, btrfs_cmd_group_info, {
-		{ "subvolume", cmd_subvolume, NULL, &subvolume_cmd_group, 0 },
-		{ "filesystem", cmd_filesystem, NULL, &filesystem_cmd_group, 0 },
-		{ "balance", cmd_balance, NULL, &balance_cmd_group, 0 },
-		{ "device", cmd_device, NULL, &device_cmd_group, 0 },
-		{ "scrub", cmd_scrub, NULL, &scrub_cmd_group, 0 },
-		{ "check", cmd_check, cmd_check_usage, NULL, 0 },
-		{ "rescue", cmd_rescue, NULL, &rescue_cmd_group, 0 },
-		{ "restore", cmd_restore, cmd_restore_usage, NULL, 0 },
-		{ "inspect-internal", cmd_inspect, NULL, &inspect_cmd_group, 0 },
-		{ "property", cmd_property, NULL, &property_cmd_group, 0 },
-		{ "send", cmd_send, cmd_send_usage, NULL, 0 },
-		{ "receive", cmd_receive, cmd_receive_usage, NULL, 0 },
-		{ "quota", cmd_quota, NULL, &quota_cmd_group, 0 },
-		{ "qgroup", cmd_qgroup, NULL, &qgroup_cmd_group, 0 },
-		{ "replace", cmd_replace, NULL, &replace_cmd_group, 0 },
-		{ "help", cmd_help, cmd_help_usage, NULL, 0 },
-		{ "version", cmd_version, cmd_version_usage, NULL, 0 },
-		NULL_CMD_STRUCT
+		&cmd_struct_subvolume,
+		&cmd_struct_filesystem,
+		&cmd_struct_balance,
+		&cmd_struct_device,
+		&cmd_struct_scrub,
+		&cmd_struct_check,
+		&cmd_struct_rescue,
+		&cmd_struct_restore,
+		&cmd_struct_inspect,
+		&cmd_struct_property,
+		&cmd_struct_send,
+		&cmd_struct_receive,
+		&cmd_struct_quota,
+		&cmd_struct_qgroup,
+		&cmd_struct_replace,
+		&cmd_struct_help,
+		&cmd_struct_version,
+		NULL
 	},
 };
 
@@ -276,6 +366,16 @@ int main(int argc, char **argv)
 
 	if (!strcmp(bname, "btrfsck")) {
 		argv[0] = "check";
+#ifdef ENABLE_BOX
+	} else if (!strcmp(bname, "mkfs.btrfs")) {
+		return mkfs_main(argc, argv);
+	} else if (!strcmp(bname, "btrfs-image")) {
+		return image_main(argc, argv);
+	} else if (!strcmp(bname, "btrfs-convert")) {
+		return convert_main(argc, argv);
+	} else if (!strcmp(bname, "btrfstune")) {
+		return btrfstune_main(argc, argv);
+#endif
 	} else {
 		int shift;
 
@@ -299,7 +399,7 @@ int main(int argc, char **argv)
 
 	fixup_argv0(argv, cmd->token);
 
-	ret = cmd->fn(argc, argv);
+	ret = cmd_execute(cmd, argc, argv);
 
 	btrfs_close_all_devices();
 
